import pathlib

from pelican import settings

thisdir = pathlib.Path(__file__).parent

AUTHOR = "Mrs. Joanne Cunningham"
SITENAME = "Nostrum eaque delectus ..."
SITEURL = ""

PATH = "content"
ARTICLE_PATHS = [
    "articles",
]

TIMEZONE = "America/New_York"

DEFAULT_LANG = "en"

# Feed generation is usually not desired when developing
FEED_ALL_ATOM = None
CATEGORY_FEED_ATOM = False
TRANSLATION_FEED_ATOM = None
AUTHOR_FEED_ATOM = None
AUTHOR_FEED_RSS = None

# Blogroll
LINKS = (
    ("Pelican", "https://getpelican.com/"),
    ("Python.org", "https://python.org/"),
)

DEFAULT_PAGINATION = 10

STATIC_PATHS = []

PLUGINS = [
    "show_source",
]

THEME = "notmyidea"  # default Pelican theme

THEME_TEMPLATES_OVERRIDES = [
    thisdir / "templates",
]
THEME_TEMPLATES_OVERRIDES = [
    thisdir / "templates",
]

pconf = settings.DEFAULT_CONFIG["PCONF"]  # set in test_show_source.py
SHOW_SOURCE_PRESERVE_EXTENSION = pconf.SHOW_SOURCE_PRESERVE_EXTENSION
SHOW_SOURCE_ON_SIDEBAR = pconf.SHOW_SOURCE_ON_SIDEBAR
SHOW_SOURCE_IN_SECTION = pconf.SHOW_SOURCE_IN_SECTION
SHOW_SOURCE_ALL_POSTS = pconf.SHOW_SOURCE_ALL_POSTS



import logging
import os
from urllib.parse import urljoin

from pelican import signals
from pelican.utils import pelican_open

logger = logging.getLogger(__name__)
source_files = []
TYPES_TO_PROCESS = ["articles", "pages", "drafts"]


def link_source_files(generator):
    """
    Processes each article/page object and formulates copy from and copy
    to destinations, as well as adding a source file URL as an attribute.
    """
    # Get all attributes from the generator that are articles or pages
    documents = sum(
        [
            getattr(generator, attr, None)
            for attr in TYPES_TO_PROCESS
            if getattr(generator, attr, None)
        ],
        [],
    )

    preserve_ext = generator.settings.get("SHOW_SOURCE_PRESERVE_EXTENSION", False)

    # Work on each item
    for post in documents:
        # condition to show a post
        if not (
            generator.settings.get("SHOW_SOURCE_ALL_POSTS")
            or post.metadata.get("show_source")
        ):
            logger.debug("show_source: sources not shown, aborting plugin")
            continue

        # Source file name can be optionally set in config
        show_source_filename = generator.settings.get(
            "SHOW_SOURCE_FILENAME", "{}.txt".format(post.slug)
        )
        try:
            # Get the full path to the original source file
            source_out = os.path.join(post.settings["OUTPUT_PATH"], post.save_as)

            # Get the path to the original source file
            source_out_path = os.path.split(source_out)[0]

            # Create 'copy to' destination for writing later
            copy_to = os.path.join(source_out_path, show_source_filename)

            # Add file to published path
            source_url = urljoin(post.save_as, show_source_filename)
        except Exception:
            logger.error(
                "show_source: Error processing source file for post", exc_info=True
            )

        # Preserve extension, if requested
        if preserve_ext:
            __, source_ext = os.path.splitext(post.source_path)

            copy_to_plain_name, __ = os.path.splitext(copy_to)
            copy_to = copy_to_plain_name + source_ext

            source_url_plain_name, __ = os.path.splitext(source_url)
            source_url = source_url_plain_name + source_ext

        # Format post source dict & populate
        out = {"copy_raw_from": post.source_path, "copy_raw_to": copy_to}

        logger.debug("Show Source: Will copy %s to %s", post.source_path, copy_to)
        source_files.append(out)
        # Also add the source path to the post as an attribute
        post.show_source_url = source_url


def _copy_from_to(from_file, to_file):
    """
    A very rough and ready copy from / to function.
    """
    with pelican_open(from_file) as text_in:
        encoding = "utf-8"
        with open(to_file, "w", encoding=encoding) as text_out:
            text_out.write(text_in)
            logger.info("show_source: Writing %s", to_file)


def write_source_files(*args, **kwargs):
    """
    Called by the `page_writer_finalized` signal to process source files.
    """
    for source in source_files:
        _copy_from_to(source["copy_raw_from"], source["copy_raw_to"])


def register():
    """
    Calls the shots, based on signals
    """
    signals.article_generator_finalized.connect(link_source_files)
    signals.page_generator_finalized.connect(link_source_files)
    signals.page_writer_finalized.connect(write_source_files)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pelican',
 'pelican.plugins.show_source',
 'pelican.plugins.test',
 'pelican.plugins.test.data']

package_data = \
{'': ['*'], 'pelican.plugins.test.data': ['content/articles/*', 'templates/*']}

install_requires = \
['pelican>=4.5']

extras_require = \
{'markdown': ['markdown>=3.2']}

setup_kwargs = {
    'name': 'pelican-show-source',
    'version': '1.1.0',
    'description': 'Pelican plugin that adds a link to post source content',
    'long_description': 'Show Source: A Plugin for Pelican\n=================================\n\n[![Build Status](https://img.shields.io/github/workflow/status/pelican-plugins/show-source/build)](https://github.com/pelican-plugins/show-source/actions)\n[![PyPI Version](https://img.shields.io/pypi/v/pelican-show-source)](https://pypi.org/project/pelican-show-source/)\n![License](https://img.shields.io/pypi/l/pelican-show-source?color=blue)\n\nThis Pelican plugin allows you to place a link to your posts’ source content files in the same way that [Sphinx][] does. It works for both pages and articles.\n\nInstallation\n------------\n\nThis plugin can be installed via:\n\n    python -m pip install pelican-show-source\n\nFor more detailed plugin installation instructions, please refer to the [Pelican Plugin Documentation][].\n\nConfiguration\n-------------\n\nTo enable the plugin, ensure that you have `SHOW_SOURCE_ON_SIDEBAR = True` or `SHOW_SOURCE_IN_SECTION = True` in your settings file.\n\n`SHOW_SOURCE_ALL_POSTS = True` will show the source for all posts, instead of only posts with the source explicitly made available (see below.) By default, this is `False`.\n\nThe option `SHOW_SOURCE_PRESERVE_EXTENSION` (`False` by default) controls the extension of the output source file. If `True`, the file will have the same extension as the real source document (`.rst`, `.md`, etc). Otherwise, it will be a plain-text `.txt` file.\n\nMaking Source Available for Posts\n---------------------------------\n\nIn order to mark posts so that their source may be seen, use the following metadata fields (unless overridden) for reStructuredText documents:\n\n```rst\n:show_source: True\n```\n\nAlternatively, for Markdown syntax:\n\n```markdown\nShow_source: True\n```\n\nThe plugin will render your source document URL to a corresponding `article.show_source_url` (or `page.show_source_url`) attribute, which is then accessible in the site templates.\n\nShow Source in the Templates\n----------------------------\n\nTo get the “show source” links to display in the article or page you will have to modify your theme, either as a sidebar display or at the foot of an article.\n\n### Article or Page Sidebar Display\n\nHow to get the source link to appear in the sidebar using the [pelican-bootstrap3][] theme:\n\n```html\n{% if SHOW_SOURCE_ON_SIDEBAR %}\n    {% if (article and article.show_source_url) or (page and page.show_source_url) %}\n        <li class="list-group-item"><h4><i class="fa fa-tags fa-file-text"></i><span class="icon-label">This Page</span></h4>\n            <ul class="list-group">\n                <li class="list-group-item">\n                    {% if article %}\n                    <a href="{{ SITEURL }}/{{ article.show_source_url }}">Show source</a>\n                    {% elif page %}\n                    <a href="{{ SITEURL }}/{{ page.show_source_url }}">Show source</a>\n                    {% endif %}\n                </li>\n            </ul>\n        </li>\n    {% endif %}\n{% endif %}\n```\n\n### Article Footer Display\n\nFollowing is some code (yes, [pelican-bootstrap3][] again) to enable a source link at the bottom of an article:\n\n```html\n{% if SHOW_SOURCE_IN_SECTION %}\n    {% if article and article.show_source_url %}\n    <section class="well" id="show-source">\n        <h4>This Page</h4>\n        <ul>\n            <a href="{{ SITEURL }}/{{ article.show_source_url }}">Show source</a>\n        </ul>\n    </section>\n    {% endif %}\n{% endif %}\n```\n\nOverriding Default Plugin Behaviour\n-----------------------------------\n\nThe default behaviour of the plugin is that revealing source is enabled on a case-by-case basis. This can be changed by the use of `SHOW_SOURCE_ALL_POSTS = True` in the settings file. This does mean that the plugin will publish all source documents no matter whether `show_source` is set in the metadata or not.\n\nUnless overridden, each document is saved as the article or page slug attribute with a `.txt` extension.\n\nSo for example, if your configuration had `ARTICLE_SAVE_AS` configured like so:\n\n```python\nARTICLE_SAVE_AS = "posts/{date:%Y}/{date:%m}/{slug}/index.html"\n```\n\n… your static HTML post and source text document will be like the following:\n\n```text\nposts/2016/10/welcome-to-my article/index.html\nposts/2016/10/welcome-to-my article/welcome-to-my article.txt\n```\n\nYou can add the `SHOW_SOURCE_FILENAME` variable in your settings file to override the source file name, so you could set the following:\n\n```python\nSHOW_SOURCE_FILENAME = "my_source_file.txt"\n```\n\nSo with the `ARTICLE_SAVE_AS` configured as above, the files would be saved\nthus:\n\n```text\nposts/2016/10/welcome-to-my article/index.html\nposts/2016/10/welcome-to-my article/my_source_file.txt\n```\n\nThis is the same behaviour for pages as well.\n\nContributing\n------------\n\nContributions are welcome and much appreciated. Every little bit helps. You can contribute by improving the documentation, adding missing features, and fixing bugs. You can also help out by reviewing and commenting on [existing issues][].\n\nTo start contributing to this plugin, review the [Contributing to Pelican][] documentation, beginning with the **Contributing Code** section.\n\n[existing issues]: https://github.com/pelican-plugins/show-source/issues\n[Contributing to Pelican]: https://docs.getpelican.com/en/latest/contribute.html\n\nLicense\n-------\n\nThis project is licensed under the AGPL-3.0 license.\n\n\n[Pelican Plugin Documentation]: https://docs.getpelican.com/en/latest/plugins.html\n[Sphinx]: https://www.sphinx-doc.org/\n[pelican-bootstrap3]: https://github.com/getpelican/pelican-themes/tree/master/pelican-bootstrap3\n',
    'author': 'Pelican Dev Team',
    'author_email': 'authors@getpelican.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/pelican-plugins/show-source',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.6.2,<4.0',
}


setup(**setup_kwargs)



import collections
import enum
import itertools
import pathlib

import pytest

import pelican


# makes easier to handle flags
class Flags(enum.IntEnum):
    SHOW_SOURCE_PRESERVE_EXTENSION = 0
    SHOW_SOURCE_ALL_POSTS = 1
    SHOW_SOURCE_ON_SIDEBAR = 2
    SHOW_SOURCE_IN_SECTION = 3


Pconf = collections.namedtuple("Pconf", [f.name for f in Flags])


@pytest.fixture(scope="module")
def site(request, tmp_path_factory):
    from pelican import settings

    # this will pass the test configuration down to the pelicanconf.py
    settings.DEFAULT_CONFIG["PCONF"] = Pconf(*request.param)

    # renders a simple site with three pages
    output = tmp_path_factory.mktemp("site")
    datadir = pathlib.Path(__file__).parent / "data"
    args = [
        datadir / "content",
        "-o",
        output,
        "-s",
        datadir / "pelicanconf.py",
        "--relative-urls",
        "--debug",
    ]
    pelican.main([str(a) for a in args])

    # write a file with a name like (for troubleshooting):
    #  PRESERVE_EXTENSION=0-ALL_POSTS=1-ON_SIDEBAR=0-IN_SECTION=1
    (
        output
        / "-".join(f"{k[12:]}={int(v)}" for k, v in zip(Pconf._fields, request.param))
    ).touch()
    yield output


FLAGS = [
    (
        p,
        False,
        True,
        False,
    )
    for p, in sorted(itertools.product([True, False], repeat=1))
]


@pytest.mark.parametrize("site", FLAGS, indirect=["site"])
def test_preserve_extension(site):
    """Test that SHOW_SOURCE_PRESERVE_EXTENSION writes the correct file extension."""
    from pelican import settings

    pconf = settings.DEFAULT_CONFIG["PCONF"]

    tag = "esse-quam-laboriosam-at-accusantium"

    # verify extension flags
    src_extension = ".md" if pconf.SHOW_SOURCE_PRESERVE_EXTENSION else ".txt"

    # verify the page has been rendered
    assert (site / (tag + ".html")).exists()
    # verify the original source has been transferred
    assert (site / (tag + src_extension)).exists()


FLAGS = [
    (p, q, r, s) for p, q, r, s in sorted(itertools.product([True, False], repeat=4))
]


@pytest.mark.parametrize("site", FLAGS, indirect=["site"])
def test_site(site):
    """Test use of all possible flags."""
    from pelican import settings

    pconf = settings.DEFAULT_CONFIG["PCONF"]

    tag = "esse-quam-laboriosam-at-accusantium"
    src_extension = ".md" if pconf.SHOW_SOURCE_PRESERVE_EXTENSION else ".txt"

    # files have the inline show source?
    has_insection = []
    for e in ["", "-2", "-3"]:
        subtxt = f'<a href="./{tag}{e}{src_extension}">Show source</a>'
        has_insection.append(subtxt in (site / (tag + f"{e}.html")).read_text())

    # files have the in sidebar show source?
    has_sidebar = []
    for e in ["", "-2", "-3"]:
        subtxt = f'(<a href="./{tag}{e}{src_extension}">.. show source</a>)'
        has_sidebar.append(subtxt in (site / (tag + f"{e}.html")).read_text())

    if not (pconf.SHOW_SOURCE_IN_SECTION or pconf.SHOW_SOURCE_ON_SIDEBAR):
        assert not any(has_insection)
        assert not any(has_sidebar)

    if not pconf.SHOW_SOURCE_IN_SECTION:
        assert not any(has_insection)

    if not pconf.SHOW_SOURCE_ON_SIDEBAR:
        assert not any(has_sidebar)

    if pconf.SHOW_SOURCE_ALL_POSTS:
        if pconf.SHOW_SOURCE_IN_SECTION:
            assert all(has_insection)
        if pconf.SHOW_SOURCE_ON_SIDEBAR:
            assert all(has_sidebar)
    else:
        if pconf.SHOW_SOURCE_IN_SECTION:
            assert has_insection == [True, False, True]
        if pconf.SHOW_SOURCE_ON_SIDEBAR:
            assert has_sidebar == [True, False, True]
